/****************************/
/* The Radicalisation Model */
/****************************/

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <math.h>
#include <time.h>

#define MAX_TIME 260
#define NO_PEOPLE 500
#define NO_SETTINGS 56+NO_PEOPLE
#define NO_SETTING_TYPES 10
#define NO_BEHAVIOURS 2
#define E 2.71828
#define alpha 1	//model parameter for activity field generation
#define beta 1	//model parameter for activity field generation
#define tau1 0.0089	//time threshold for emergence
#define tau2 0	//time threshold for exposure
#define epsilon 0.1	//propensity threshold for emergence

using namespace std;

class Setting {
public:
	string Name;
	string Type;		//e.g. youth club, school, leisure centre etc
	float xLocation;	//physical coordinates
	float yLocation;	//physical coordinates
	int Size;			//capacity in terms of no of people, or equivalent
	float CollectiveEfficacy;	//a measure of the collective efficacy of the setting (read from input file)
	float Radicalisation[NO_BEHAVIOURS];		//how radicalising is the setting
	float Religion;	//mean religion of those visiting the setting
	float Propensity; //mean propensity of those visiting the setting
	float SelfControl;	//mean self-control of those visiting the setting
	float SPI;	//mean SPI of those visiting the setting
	float Age; 	//mean age of those visiting the setting
	
};	

class ActivityField {	// a table for each person showing how long they spend where
public:
	string Places[NO_SETTINGS];
	double TimeSpent[NO_SETTINGS];
	friend class Person;
};

class Person {
public:
	int Age;
	string Religion;
	string Occupation;		//choice of Employed, Student or Unemployed
	int Gender;			//1 for male, 0 for female
	float xHomeLocation;	//physical coordinates
	float yHomeLocation;	//physical coordinates
	float SPI;	//based on RPI, distributed normally with varying mean and s.d.=1
	float SelfControl;	//immutable, distributed N(0,1)
	double Propensity[NO_BEHAVIOURS];
	double OldPropensity;
	ActivityField AF1;
	void ComputeAF (Setting *, string *);
	float SelectionQuotient[NO_SETTINGS];
	void ComputeSelectionQ (Setting *, string *, int *);
	double Exp[NO_BEHAVIOURS];	//the raw exposure to all radicalising settings

};

//A look-up table defining the selection quotient for each setting, determined by the setting, and the person's age, religion & occupation - HAPPY WITH THIS
void Person::ComputeSelectionQ (Setting place[], string types[], int cts[]) {
	int j, i, counts[NO_SETTINGS];
	double remaining_hours, sum=0;
	
	for(j=0;j<NO_SETTINGS;j++){
		SelectionQuotient[j]=0;
		
		if((xHomeLocation==place[j].xLocation) && (yHomeLocation==place[j].yLocation))
			place[j].Type="\"Home\"";
		counts[j]=0;
		for(i=0;i<NO_SETTING_TYPES;i++){
			if(place[j].Type==types[i]) {
				counts[j]=cts[i];
				break;
			}
		}
		
		if((Age<18) && (Occupation=="\"Student\"") && (place[j].Type=="\"School\""))
			SelectionQuotient[j]=35;
		if((Age>=18) && (Occupation=="\"Student\"") && (place[j].Type=="\"University\""))
			SelectionQuotient[j]=40;
		if((Occupation=="\"Employed\"") && (place[j].Type=="\"Office\""))
			SelectionQuotient[j]=40;
		if((Religion=="\"Christian\"") && (place[j].Type=="\"Church\""))
			SelectionQuotient[j]=2;
		if((Religion=="\"Muslim\"") && (place[j].Type=="\"Mosque\""))
			SelectionQuotient[j]=2;
	}
	
	for(j=0;j<NO_SETTINGS;j++)
		sum+=(SelectionQuotient[j]/counts[j]);
	
	remaining_hours=112-sum;
	
	if(Age<20){
		for(j=0;j<NO_SETTINGS;j++)
			if((place[j].Type=="\"Home\"") || (place[j].Type=="\"Residence\"") || (place[j].Type=="\"YouthClub\"") || (place[j].Type=="\"HighStreet\"") || (place[j].Type=="\"LeisureCentre\""))
				SelectionQuotient[j]=remaining_hours/5;
	}
	if(Age>=20){
		for(j=0;j<NO_SETTINGS;j++)
			if((place[j].Type=="\"Home\"") || (place[j].Type=="\"Residence\"") || (place[j].Type=="\"HighStreet\"") || (place[j].Type=="\"LeisureCentre\""))
				SelectionQuotient[j]=remaining_hours/4;
	}
	
	//Check the adding up!
	sum=0;
	for(j=0;j<NO_SETTINGS;j++)
		sum+=(SelectionQuotient[j]/counts[j]);
	
	if(sum>112.1 || sum<111.9){
		cout<<"Incorrect Selection Quotient calculation! sum="<<sum<<endl;
		exit(1);
	}
	
	for(j=0;j<NO_SETTINGS;j++)
		SelectionQuotient[j]/=112;
	
	for(j=0;j<NO_SETTINGS;j++)
		if((xHomeLocation==place[j].xLocation) && (yHomeLocation==place[j].yLocation))
			place[j].Type="\"Residence\"";
		
	return;
	
}

//Calculates the person's activity field
void Person::ComputeAF (Setting place[], string types[]) {
	double c[NO_SETTINGS];
	int j, k, topworkplace=0, rel, code, b=0;
	double A_i[]={0,0,0,0,0,0,0,0,0,0,0};	// one scaling factor for each setting subtype
	double temp=0, W[NO_SETTINGS], Similarity[NO_SETTINGS];
	double Difference_rel, Difference_p, Difference_sc, Difference_spi, Difference_age;
	
	for(j=0;j<NO_SETTINGS;j++)
		AF1.Places[j]=place[j].Name;
	
	if(Religion=="\"Christian\"") rel=0;
	else rel=1;
	if(Age<16) code=0;
	if((Age>=16) && (Age<=18)) code=1;
	if((Age>=19) && (Age<=23)) code=2;
	if((Age>=24) && (Age<=30)) code=3;
	if((Age>=31) && (Age<=40)) code=4;
	if(Age>40) code=5;
	
	// Define the similarity function
	for(j=0;j<NO_SETTINGS;j++){
		Difference_rel=rel-place[j].Religion;
		if(Difference_rel<0) Difference_rel*=-1;
		Difference_p=Propensity[b]-place[j].Propensity;
		if(Difference_p<0) Difference_p*=-1;
		Difference_sc=SelfControl-place[j].SelfControl;
		if(Difference_sc>0) Difference_sc*=-1;
		Difference_sc=1-pow(E,Difference_sc);
		Difference_spi=SPI-place[j].SPI;
		if(Difference_spi>0) Difference_spi*=-1;
		Difference_spi=1-pow(E,Difference_spi);
		Difference_age=code-place[j].Age;
		if(Difference_age<0) Difference_age*=-1;
		Difference_age/=5;
		Similarity[j]=Difference_rel+Difference_p+Difference_sc+Difference_spi+Difference_age;
		Similarity[j]/=5;
		Similarity[j]=1-Similarity[j]; //now is between 0 (for v different) and 1 (for identical)
	}
		
	for(j=0;j<NO_SETTINGS;j++){
		W[j]=place[j].Size*Similarity[j];	//this makes the setting more attractive if the morality of the person is closer to the morality of the place
		W[j]=pow(W[j],alpha);
		
		c[j]=(place[j].yLocation-yHomeLocation)*(place[j].yLocation-yHomeLocation) + (place[j].xLocation-xHomeLocation)*(place[j].xLocation-xHomeLocation);
		c[j]=0-sqrt(c[j]);
		
		AF1.TimeSpent[j]=SelectionQuotient[j] * W[j] * pow(E,beta*c[j]);
	}
	
	for(j=0;j<NO_SETTINGS;j++)
		if((xHomeLocation==place[j].xLocation) && (yHomeLocation==place[j].yLocation))
			place[j].Type="\"Home\"";
	
	for(j=0;j<NO_SETTINGS;j++){
		for(k=0;k<NO_SETTING_TYPES;k++) if(types[k]==place[j].Type) {
			A_i[k]+=(W[j] * pow(E,beta*c[j]));
			break;
		}
	}
	
	//Error check
	for(k=0;k<NO_SETTING_TYPES;k++) if(A_i[k]==0){
		cout<<"Error in activity field generation!\n";
		exit(1);
	}
	
	for(j=0;j<NO_SETTINGS;j++){
		for(k=0;k<NO_SETTING_TYPES;k++) if(types[k]==place[j].Type){
			AF1.TimeSpent[j]/=A_i[k];
			break;
		}
	}
	
	for(j=0;j<NO_SETTINGS;j++)
		if((xHomeLocation==place[j].xLocation) && (yHomeLocation==place[j].yLocation))
			place[j].Type="\"Residence\"";
	
	//Identify the workplace where they spend the maximum amount of time and make that their sole workplace
	for(j=0;j<NO_SETTINGS;j++)
		if((place[j].Type=="\"Office\"") || (place[j].Type=="\"University\"") || (place[j].Type=="\"School\"")){
			if(AF1.TimeSpent[j]>temp){
				topworkplace=j;
				temp=AF1.TimeSpent[j];
			}
		}
	
	for(j=0;j<NO_SETTINGS;j++)
		if((place[j].Type=="\"Office\"") || (place[j].Type=="\"University\"") || (place[j].Type=="\"School\""))
			AF1.TimeSpent[j]=0;
	AF1.TimeSpent[topworkplace]=SelectionQuotient[topworkplace];
	
}

Person Exposure (Person bob, Setting allplaces[]) {
	
	double ln_mu;
	int b=0;	//can generalise to all b's later
	
	ln_mu = -6.91 + (0.9*bob.SPI) - (0.45*bob.SelfControl) + (0.05*bob.SPI*bob.SelfControl) + (0.028*bob.Exp[b]);
	bob.Propensity[b]=1/(1+0.1228*pow(E,ln_mu));
	bob.Propensity[b]=pow(bob.Propensity[b],8.143322);
	bob.Propensity[b]=1-bob.Propensity[b];
	
	return bob;
	
}

Setting Emergence (Setting place, Person allpeople[]) {
	float sum, threshold;
	int b=0, i, j, counter;
	
		sum=0;
		counter=0;
		for(i=0;i<NO_PEOPLE;i++){
			for(j=1;j<NO_SETTINGS;j++){ //exclude prison
				threshold=epsilon/place.CollectiveEfficacy;
				if(place.Name == allpeople[i].AF1.Places[j]){
					if(allpeople[i].AF1.TimeSpent[j]>tau1){
						if(allpeople[i].Propensity[b]>threshold){	//identify the place in each person's activity field then see if they spend enough time there
							counter++;
							sum+=(allpeople[i].Propensity[b]);
							}
						}	//if so add the person's propensity
					break;
					}	//then do the same for the next person
			}
		}
		if(counter!=0) place.Radicalisation[b]=sum*pow(place.CollectiveEfficacy,0.5)/counter;
		if(counter==0) place.Radicalisation[b]=0;
	
	return place;
	
}

void IdentifyFriends (Person allpeople[]){
	int s, i, j, bestfriend[NO_PEOPLE];
	double homophily[NO_PEOPLE][NO_PEOPLE], temp;
	
	for(i=0;i<NO_PEOPLE;i++) for(j=0;j<NO_PEOPLE;j++){
		homophily[i][j]=0;
		for(s=0;s<(NO_SETTINGS-NO_PEOPLE);s++){ //so homophily is the difference in activity fields except residences
			if(allpeople[i].AF1.TimeSpent[s]>allpeople[j].AF1.TimeSpent[s])
				homophily[i][j]+=(allpeople[i].AF1.TimeSpent[s]-allpeople[j].AF1.TimeSpent[s]);
			else
				homophily[i][j]+=(allpeople[j].AF1.TimeSpent[s]-allpeople[i].AF1.TimeSpent[s]);
		}
	}
	
	//fill in remainder of homophily matrix
	for(i=0;i<NO_PEOPLE;i++) for(j=i+1;j<NO_PEOPLE;j++){
		homophily[i][j]=homophily[j][i];
	}
	for(i=0;i<NO_PEOPLE;i++)
		homophily[i][i]=999999;
	
	//then person j is best friend of person i if they have the lowest homophily rating
	for(i=0;i<NO_PEOPLE;i++){
		temp=999999;
		for(j=0;j<NO_PEOPLE;j++)
			if(homophily[i][j]<temp){
				bestfriend[i]=j;
				temp=homophily[i][j];
			}
	}
	
	//therefore they spend time at that person's house rather than anyone else's
	for(i=0;i<NO_PEOPLE;i++)
		for(j=0;j<NO_PEOPLE;j++)
			if(i!=j)
				allpeople[i].AF1.TimeSpent[NO_SETTINGS-NO_PEOPLE+j]=0;
	
	for(i=0;i<NO_PEOPLE;i++)
		allpeople[i].AF1.TimeSpent[NO_SETTINGS-NO_PEOPLE+bestfriend[i]]=allpeople[i].SelectionQuotient[NO_SETTINGS-NO_PEOPLE+bestfriend[i]];
	
}

void FileError (char file[])
{
	cout<<"The "<<file<<" file could not be opened.\n";
	cout<<"Possible errors:\n";
	cout<<"1. The file does not exist\n";
	cout<<"2. The path was not found\n";
	exit(1);
	
}

int main (int argc, char * const argv[]) 
{
	int t, b, years, counter, code;
	long i, j;
	double r, r1, p_rand, p_rand1, meanExp, varExp;
	char file1[] = “../data/people.csv";
	char file2[] = “../data/settings.csv";
	char file3[] = “../results/people_out.csv";
	char file4[] = “../results/settings_out.csv";
	ifstream inputfile;
	ofstream outputfile;
	ostringstream convert;
	Person pers[NO_PEOPLE];
	Setting places[NO_SETTINGS];
	string SubTypes[] = {"\"Office\"","\"University\"","\"School\"","\"Church\"","\"Mosque\"","\"HighStreet\"","\"YouthClub\"","\"LeisureCentre\"","\"Residence\"","\"Home\""};
	int Counts[] = {0,0,0,0,0,0,0,0,-1,1};
	int CheckSum=0;
	//	time_t seconds;
	
	inputfile.open(file1, ios::in);
	if(inputfile.fail()) FileError(file1);
	else
	{		
		for(i=0;i<NO_PEOPLE;i++){
			inputfile>>pers[i].Age;
			inputfile>>pers[i].Religion;
			inputfile>>pers[i].Occupation;
			inputfile>>pers[i].Gender;			
			inputfile>>pers[i].xHomeLocation;
			inputfile>>pers[i].yHomeLocation;
			for(b=0;b<NO_BEHAVIOURS;b++)
				inputfile>>pers[i].Propensity[b];
		}
		
		inputfile.close();
		if(inputfile.fail())
		{
			cout<<"The "<<file1<<" file could not be closed.\n";
			exit(1);
		}
	}
	
	inputfile.open(file2, ios::in);
	if(inputfile.fail()) FileError(file2);
	else
	{
		for(i=0;i<(NO_SETTINGS-NO_PEOPLE);i++){
			inputfile>>places[i].Name;
			inputfile>>places[i].Type;
			inputfile>>places[i].xLocation;
			inputfile>>places[i].yLocation;
			inputfile>>places[i].Size;
			inputfile>>places[i].CollectiveEfficacy;
		}
		
		// Add people's homes to the settings list
		for(i=(NO_SETTINGS-NO_PEOPLE);i<NO_SETTINGS;i++){
			convert.str("");
			convert << 1+i-(NO_SETTINGS-NO_PEOPLE);
			places[i].Name="\"House"+convert.str()+"\"";
			places[i].Size=4;	//Average household size
			places[i].xLocation=pers[i-(NO_SETTINGS-NO_PEOPLE)].xHomeLocation;
			places[i].yLocation=pers[i-(NO_SETTINGS-NO_PEOPLE)].yHomeLocation;
			places[i].Type="\"Residence\"";
			places[i].CollectiveEfficacy=1;
		}
		
		inputfile.close();
		if(inputfile.fail())
		{
			cout<<"The "<<file2<<" file could not be closed.\n";
			exit(1);
		}
	}
	
	//Error check on the Size attribute
	for(i=0;i<NO_SETTINGS;i++)
		if(places[i].Size==0)
		{
			cout<<"Error! "<<places[i].Name<<" has size=0!\n";
			exit(1);
		}
	
	//Set certain attributes at the start to their initial settings
	for(j=0;j<NO_SETTINGS;j++) for(b=0;b<NO_BEHAVIOURS;b++){
		places[j].Radicalisation[b]=0;
	}
	for(j=0;j<NO_SETTINGS;j++){
		places[j].Religion=0.5;
		places[j].Propensity=0.5;
		places[j].SelfControl=0;
		places[j].SPI=0;
		places[j].Age=2;
	}
	for(i=0;i<NO_PEOPLE;i++) pers[i].OldPropensity=0;
	
	//Define SelfControl based on a standard normal distribution across the whole population
	for(i=0;i<NO_PEOPLE;i++){
		r1=rand();
		r=rand();
		p_rand=(float)r/((float)RAND_MAX);
		p_rand1=(float)r/((float)RAND_MAX);
		pers[i].SelfControl = sqrt(-2.0 * log(p_rand)) * cos(6.28318531 * p_rand1);
	}
	
	//Define SPI based on a standard normal distribution across the whole population
	//but with the mean varying according to gender and age (1=male, 0=female).
	//The 0.3636 is the difference in means between male & female people at each age group.
	//The normal distribution generated via the Box-Muller method.
	
	//time(&seconds);
	//srand((unsigned int)seconds);   //The RNG seeding is switched off during testing so it's always the same input
	for(i=0;i<NO_PEOPLE;i++){
		r1=rand();
		r=rand();
		p_rand=(float)r/((float)RAND_MAX);
		p_rand1=(float)r/((float)RAND_MAX);
		pers[i].SPI = sqrt(-2.0 * log(p_rand)) * cos(6.28318531 * p_rand1);
		if(pers[i].Age<=14)
			pers[i].SPI += (1.6727+(pers[i].Gender*0.3636));
		if(pers[i].Age==15)
			pers[i].SPI += (1.1745+(pers[i].Gender*0.3636));
		if(pers[i].Age==16)
			pers[i].SPI += (0.6764+(pers[i].Gender*0.3636));
		if(pers[i].Age==17)
			pers[i].SPI += (0.1782+(pers[i].Gender*0.3636));
		if(pers[i].Age>=18)
			pers[i].SPI += (-0.32+(pers[i].Gender*0.3636));
	}
	
	 //make one person extra vulnerable to force exposure / emergence to happen
	 pers[471].SPI+=1;

		
	//The main section of the model
	b=0;
	//Count how many of each Setting Type to calculate Selection Quotients
	for(j=0;j<NO_SETTINGS;j++) for(i=0;i<NO_SETTING_TYPES;i++)
		if(places[j].Type == SubTypes[i]) Counts[i]++;
	
	for(i=0;i<NO_SETTING_TYPES;i++) 
		CheckSum+=Counts[i];
	if(CheckSum!=NO_SETTINGS){
		cout<<"Error! Unknown Setting Type in "<<file2<<"!\n"<<endl;
		exit(1);}
	
	// Set up output files...
	outputfile.open(file3, ios::out);
	if(outputfile.fail()) FileError(file3);
	else {
		outputfile<<"Time";
		for(i=0;i<NO_PEOPLE;i++)
			outputfile<<"\tPerson "<<i+1;
		outputfile<<endl;
		outputfile<<"0";
		for(i=0;i<NO_PEOPLE;i++)
			outputfile<<"\t"<<pers[i].Propensity[b];
		outputfile<<endl;		
		outputfile.close();
		if(outputfile.fail()) {
			cout<<"The "<<file3<<" file could not be closed.\n";
			exit(1);
		}
	}
	
	outputfile.open(file4, ios::out);
	if(outputfile.fail()) FileError(file4);
	else {
		outputfile<<"Time";
		for(i=0;i<NO_SETTINGS;i++)
			outputfile<<"\t"<<places[i].Name;
		outputfile<<endl;
		outputfile<<"0";
		for(i=0;i<NO_SETTINGS;i++)
			outputfile<<"\t"<<places[i].Radicalisation[0];
		outputfile<<endl;		
		outputfile.close();
		if(outputfile.fail()) {
			cout<<"The "<<file4<<" file could not be closed.\n";
			exit(1);
		} 
	}

	
	//Start the timestep loop; one tick = one week
	for(t=0;t<MAX_TIME;t++){
		cout<<MAX_TIME-t<<endl;
		
		//Increase people's ages and decrease their SPI as applicable
		for(i=0;i<NO_PEOPLE;i++){
			years=t/52;
			pers[i].Age+=years;
			if((pers[i].Age<18) && (pers[i].Age>=14))	//i.e. they are in the age bracket where SPI changes
				pers[i].SPI-=0.0095804;
		}
				
		// Calculate Selection Quotients
		for(i=0;i<NO_PEOPLE;i++)
			pers[i].ComputeSelectionQ(places, SubTypes, Counts);		
		
		//Calculate Activity Fields
		for(i=0;i<NO_PEOPLE;i++)
			pers[i].ComputeAF(places, SubTypes);
		
		//Identify each person's best friend and modify Activity Field accordingly
		IdentifyFriends(pers);
		
		//Calculate the mean religion, propensity, self-control, SPI and age of everyone visiting each setting
		for(j=0;j<NO_SETTINGS;j++){
			//Religion
			places[j].Religion=0;
			counter=0;
			for(i=0;i<NO_PEOPLE;i++){
				if(pers[i].Religion=="\"Christian\"") code=0;
				else code=1;
				if(pers[i].AF1.TimeSpent[j]>0){
					places[j].Religion+=code;
					counter++;
				}
			}
			if(counter!=0)
				places[j].Religion/=counter;
			else places[j].Religion=0.5;
			//Propensity
			places[j].Propensity=0;
			counter=0;
			for(i=0;i<NO_PEOPLE;i++){
				if(pers[i].AF1.TimeSpent[j]>0){
					places[j].Propensity+=pers[i].Propensity[b];
					counter++;
				}
			}
			if(counter!=0)
				places[j].Propensity/=counter;
			else places[j].Propensity=0.5;
			places[j].SelfControl=0;
			//Self-Control
			places[j].SelfControl=0;
			counter=0;
			for(i=0;i<NO_PEOPLE;i++){
				if(pers[i].AF1.TimeSpent[j]>0){
					places[j].SelfControl+=pers[i].SelfControl;
					counter++;
				}
			}
			if(counter!=0)
				places[j].SelfControl/=counter;
			else places[j].SelfControl=0;			
			places[j].SPI=0;
			//SPI
			places[j].SPI=0;
			counter=0;
			for(i=0;i<NO_PEOPLE;i++){
				if(pers[i].AF1.TimeSpent[j]>0){
					places[j].SPI+=pers[i].SPI;
					counter++;
				}
			}
			if(counter!=0)
				places[j].SPI/=counter;
			else places[j].SPI=0;
			//Age
			places[j].Age=0;
			counter=0;
			for(i=0;i<NO_PEOPLE;i++){
				if(pers[i].Age<16) code=0;
				if((pers[i].Age>=16) && (pers[i].Age<=18)) code=1;
				if((pers[i].Age>=19) && (pers[i].Age<=23)) code=2;
				if((pers[i].Age>=24) && (pers[i].Age<=30)) code=3;
				if((pers[i].Age>=31) && (pers[i].Age<=40)) code=4;
				if(pers[i].Age>40) code=5;
				if(pers[i].AF1.TimeSpent[j]>0){
					if(pers[i].AF1.TimeSpent[j]>0){
						places[j].Age+=code;
						counter++;
					}
				}
			}
			if(counter!=0)
				places[j].Age/=counter;
			else places[j].Age=2;
		}		
				
		// Call the Emergence function
		for(j=1;j<NO_SETTINGS;j++)
			places[j]=Emergence(places[j], pers);
		places[0].Radicalisation[b]=0;

		// Calculate Exp_b for each person
		for(i=0;i<NO_PEOPLE;i++){
			
			pers[i].Exp[b]=0;
			counter=0;
			for(j=0;j<NO_SETTINGS;j++){
				if(pers[i].AF1.TimeSpent[j]>tau2){
					pers[i].Exp[b]+=places[j].Radicalisation[b];	
					counter++;
				}
			}
			if(counter!=0)
				pers[i].Exp[b]/=counter;
						
			//Next standardise Exp[b]
			meanExp=0.001;
			varExp=0.000006;
			pers[i].Exp[b]-=meanExp;
			pers[i].Exp[b]/=sqrt(varExp);
			
		}
				
		// Call the Exposure function
		for(i=0;i<NO_PEOPLE;i++)
			pers[i]=Exposure(pers[i], places);
		
		// Write output files...
		outputfile.open(file3, ios::out|ios::app);
		if(outputfile.fail()) FileError(file3);
		else {
			outputfile<<t+1;
			for(i=0;i<NO_PEOPLE;i++)
				outputfile<<"\t"<<pers[i].Propensity[b];
			outputfile<<endl;
			outputfile.close();
			if(outputfile.fail()) {
				cout<<"The "<<file3<<" file could not be closed.\n";
				exit(1);
			}
		}
		
		outputfile.open(file4, ios::out|ios::app);
		if(outputfile.fail()) FileError(file4);
		else {
			outputfile<<t+1;
			for(i=0;i<NO_SETTINGS;i++)
				outputfile<<"\t"<<places[i].Radicalisation[b];
			outputfile<<endl;
			outputfile.close();
			if(outputfile.fail()) {
				cout<<"The "<<file4<<" file could not be closed.\n";
				exit(1);
			} 
		}
		
		//Return everyone's Age to the original before the next loop
		for(i=0;i<NO_PEOPLE;i++){
			pers[i].Age-=(t/52);
		}
		
	}
		 	
    return 0;
}
